
local ssh = {}

local schema = require("schema")
local safer = require("safer")
local localhost = require("sga.exec.localhost")

local function quote(str)
   return string.format("%q", str)
end

function ssh.read_file(self, filename)
   return self:run("cat "..quote(filename))
end

function ssh.write_file(self, filename, data)
   return self:run_write("cat > "..quote(filename), data)
end

function ssh.remove(self, filename)
   return self:run("rm "..quote(filename))
end

local function wrap_ssh(self, cmdline)
   return "ssh -p "..self.port.." "..self.host.." "..quote(cmdline)
end

function ssh.run(self, cmdline)
   return self.localhost:run(wrap_ssh(self, cmdline))
end

function ssh.run_write(self, cmdline, data)
   return self.localhost:run_write(wrap_ssh(self, cmdline), data)
end

function ssh.run_ok(self, cmdline)
   return self.localhost:run_ok(wrap_ssh(self, cmdline))
end

local function check_clock_skew(self)
   if os.time() > self.clock_skew_timestamp + 3600*24 then
      local now = tonumber((self:run("date +%s"))) or 0
      if now > 1000000000 then
         local localtime = os.time()
         self.clock_skew = now - localtime
         self.clock_skew_timestamp = localtime
      end
   end
end

function ssh.time(self)
   check_clock_skew(self)
   return os.time() + self.clock_skew
end

function ssh.new(exec_config)
   if not exec_config then
      return nil, "configuration error: driver_config.exec_config is missing."
   end
   local exec_config_schema = schema.Record {
      host = schema.String,
      port = schema.Optional(schema.NumberFrom(1,65535)),
      -- TODO add other ssh options here
   }
   local err = schema.CheckSchema(exec_config, exec_config_schema)
   if err then
      return nil, "configuration error in driver_config.exec_config: "..tostring(err)
   end

   local self = {
      host = exec_config.host,
      port = exec_config.port or 22,
      localhost = localhost.new(),
      clock_skew = 0,
      clock_skew_timestamp = 0,
   }
   setmetatable(self, { __index = ssh } )

   check_clock_skew(self)

   return safer.table(self)
end

return ssh
